PROGRAM Compare_Pk
  USE cosmo
  USE growth
  USE linearpk
  USE sigma
  USE sigmagaus
  ! A program for computing non-linear power spectra using:
  ! 1) Peacock and Dodds [Peacock and Dodds, MNRAS, 280, L19 (1996)]
  ! 2) Ma et al. [Ma et al., ApJL, 521, L1 (1999)]
  ! 3) Halofit [Smith et al., MNRAS, 341, 1311 (2003)]
  ! - k is in units of h Mpc^-1
  ! - P(k) is in units of h^-3 Mpc^3
  ! January 1, 2012: E.Komatsu
  IMPLICIT none
  character(len=128) :: filename
  integer :: n
  real :: chebev,lnsigma2
  double precision :: g,z,D,sig8,pi=3.14159d0
  double precision :: linear_pk,k_ov_h,pk,dlnk,lnk,zin
  external g,linear_pk
  double precision :: D2nl,kNL
  ! For Peacock&Dodds(1996)
  double precision :: dlnpkdlnk
  double precision :: y,A,B,alpha,beta,V 
  external dlnpkdlnk
  ! For Ma et al. (1999)
  double precision :: g0,c1,c2 
  ! For Halofit; Smith et al. (2003)
  real :: dlnsigma2dlnRh,lnRh
  double precision :: rneff,rncur,rknl
  integer :: i,iter,imax=50 
  double precision, allocatable :: Rtest(:)
  double precision :: zbrent,func
  double precision :: ltest,m1,m2,ml,mu
  external zbrent,func
! Specify three cosmological parameters
! The data type has been defined in MODULE cosmo.
  ode0=0.723d0
  om0=0.277d0
  w=-1d0
! read in and tabulate P(k)
  filename='wavenumber_pknowiggle_at_z=30.txt'
  !filename='wmap5baosn_max_likelihood_matterpower_at_z=30.dat'
  n=896 ! # of lines in the file
  zin=30d0
  CALL open_linearpk(filename,n)
! compute and fit sigma^2(R) to Chebyshev polynomials
  CALL compute_sigma2
! tabulate g(z) by calling "setup_growth"
  call setup_growth
! ask for redshift
  print*,'redshift?'
  read*,z
  D=g(z)/g(zin)*(1d0+zin)/(1d0+z) ! linear growth relative to zin
! compute sigma8 at a given redshift
  lnsigma2 = CHEBEV(lnR1,lnR2,c,ndim,alog(8.)) ! with the tophat window
  sig8=dble(exp(0.5*lnsigma2))*D
  print*,'sigma8 at z=',z,'is',sig8
! For Halofit, compute and fit sigma^2(R) WITH A GAUSSIAN WINDOW to 
! Chebyshev polynomials
  CALL compute_sigma2gaus
! For Halofit, find initial radius range for zbrent (non-lin. equation solver),
! within which Rsigma will be found.
  allocate(Rtest(0:imax))
  ! initial guess for Rsigma [h^-1 Mpc]
  Rtest(0)=3d0 
  ltest=func(dlog(Rtest(0)),D)
  if(ltest.le.0d0)then
     do i=1,imax
        Rtest(i)=Rtest(i-1)/2.d0
        ltest=func(dlog(Rtest(i)),D)
        if (ltest.gt.0d0)then
           m1=dlog(Rtest(i))
           m2=dlog(Rtest(i-1))
           exit
        endif
     enddo
  else
     do i=1,imax
        Rtest(i)=Rtest(i-1)*2.d0
        ltest=func(dlog(Rtest(i)),D)
        if(ltest.lt.0d0)then
           m1=dlog(Rtest(i))
           m2=dlog(Rtest(i-1))
           exit
        endif
     enddo
  endif
  deallocate(Rtest)
  ! Lower & upper limits for R, between which Rsigma will be found.
  ml=min(m1,m2) 
  mu=max(m1,m2)
  ! Find Rsigma [h^-1 Mpc] satisfying Sigma(Rsigma)=1 using zbrent.
  lnRh=real(zbrent(func,ml,mu,1d-4,iter,D))
  dlnsigma2dlnRh=CHEBEV(lnR1g,lnR2g,cgder,ndimg,lnRh) ! dlnSigma^2/dlnR
  rneff=-3d0-dble(dlnsigma2dlnRh)
  rncur=-dble(CHEBEV(lnR1g,lnR2g,cgderder,ndimg,lnRh)) ! -d^2lnSigma^2/dlnR^2
  rknl=dble(exp(-lnRh))
  print*,'Halofit: effective spectral quantities:'
  write(*,20) 'rknl [h/Mpc] =',rknl,'rneff=',rneff, 'rncur=',rncur
20 format(a14,f12.6,2x,a6,f12.6,2x,a6,f12.6)
! now output P(k,z) as a function of k
  open(1,file='wavenumber_pkpd96.txt',status='unknown')
  open(2,file='wavenumber_pkma99.txt',status='unknown')
  open(3,file='wavenumber_pkhalofit.txt',status='unknown')
  k_ov_h=1d-4 ! h/Mpc
  dlnk=2d-2
  do while (k_ov_h<1d2)
     !------------------- Peacock & Dodds (1996) -------------------
     y=1d0+dlnpkdlnk(0.5d0*k_ov_h)/3d0
     A=0.482d0*y**(-0.947d0)
     B=0.226d0*y**(-1.778d0)
     alpha=3.310d0*y**(-0.244d0)
     beta=0.862d0*y**(-0.287d0)
     V=11.55d0*y**(-0.423d0)
     y=k_ov_h**3d0*D**2d0*linear_pk(k_ov_h)/(2d0*pi*pi) ! lin. k^3*P(k)/(2*pi^2)
     D2nl=y*((1d0+B*beta*y+(A*y)**(alpha*beta)) & ! non-lin. k^3*P(k)/(2*pi^2)
          /(1d0+((A*y)**alpha*g(z)**3d0/(V*y**0.5))**beta))**(1d0/beta)
     kNL=k_ov_h*(1d0+D2nl)**(1d0/3d0) ! non-linear wavenumber
     pk=D2nl*(2d0*pi*pi)/kNL**3d0 ! non-linear P(k)
     write(1,'(2E18.8)')kNL,pk
     !------------------- Ma et al. (1999) -------------------
     beta=0.83d0
     c1=1.08d-4
     c2=2.10d-5
     g0=g(0d0)*dabs(w)**(1.3d0*dabs(w)-0.76d0)
     ! linear k^3*P(k)/(2*pi^2) divided by g0^1.5*sig8(z)^beta
     y=k_ov_h**3d0*D**2d0*linear_pk(k_ov_h)/(2d0*pi*pi)/g0**1.5d0/sig8**beta
     D2nl=k_ov_h**3d0*D**2d0*linear_pk(k_ov_h)/(2d0*pi*pi)&!NL k^3*P(k)/(2*pi^2)
         *(1d0+dlog(1d0+0.5d0*y))*(1d0+0.02d0*y**4d0+c1*y**8d0/g(z)**3d0) &
         /(1d0+c2*y**7.5d0)
     kNL=k_ov_h*(1d0+D2nl)**(1d0/3d0) ! non-linear wavenumber
     pk=D2nl*(2d0*pi*pi)/kNL**3d0 ! non-linear P(k)
     write(2,'(2E18.8)')kNL,pk
     !------------------ Halofit, Smith et al. (2003) ------------------
     y=k_ov_h**3d0*D**2d0*linear_pk(k_ov_h)/(2d0*pi*pi) ! lin. k^3*P(k)/(2*pi^2)
     CALL halofit(z,k_ov_h,rneff,rncur,rknl,y,D2nl)
     pk=D2nl*(2d0*pi*pi)/k_ov_h**3d0 ! non-linear P(k)
     write(3,'(2E18.8)')k_ov_h,pk
     lnk=dlog(k_ov_h)+dlnk
     k_ov_h=dexp(lnk)
  enddo
  call close_linearpk
  close(1)
  close(2)
  close(3)
END PROGRAM Compare_Pk
!-----------------------------------------------------------------------------
double precision function func(lnRh,growth)
  USE sigmagaus
! zbrent will solve the equation, sigma(Rsigma,z)=1, for ln(Rsigma).
  IMPLICIT none
  double precision, intent(IN) :: lnRh,growth ! Rh in units of h^-1 Mpc
  real :: chebev,lnsigma2
  lnsigma2=CHEBEV(lnR1g,lnR2g,cg,ndimg,real(lnRh))!ln(sigma^2) with Gaussian window
  func=0.5d0*dble(lnsigma2)+dlog(growth)-0d0 
  return
END function func
